#!/usr/bin/env python
# -*- coding: utf-8 -*
# @Date    : 2019-04-12 09:56:01
# @Author  : Blackstone
# @to      :

# coding=utf-8
import re
import time
import logging
import os
import cx_Oracle
import traceback
from . import Rule,get_connect_config,log


class DBHelper:

    _connect_pool={}
    _timeout=28800#8小时

    def __init__(self,config_path=None,**configoptions):

        self.config=get_connect_config(config_path,**configoptions)


        if not isinstance(self.config,dict):
            raise TypeError("config是字典=>",self.config)


  

    def _db_connect(self):

        

        log.debug("----------------------")
        log.debug("----尝试连接数据库-----")
        log.debug("----------------------")

        try:



            self.dbtype=self.config["dbtype"]
            self.dbname=self.config["dbname"]
            #oracle不需要这两项
            self.host=self.config["host"]
            self.port=self.config["port"]
            self.user=self.config["username"]
            self.pwd=self.config["password"]


            key="+".join([self.dbtype,self.dbname,str(self.host),str(self.port),self.user,str(self.pwd)])

            log.debug("数据库类型=>%s"%self.dbtype)

            log.debug("数据库名=>%s"%self.dbname)
            log.debug("数据库地址(仅mysql|db2)=>%s|%s"%(self.host,self.port))
            log.debug("数据库账号=>%s|%s"%(self.user,self.pwd))

            self.conn=self._connect_pool.get(key)

            need_connect=False

            if  self.conn  is not None:
                try:
                    self.conn.ping()
                except:
                    need_connect =True
            else:
                need_connect=True

             
            if need_connect==True:
                if self.dbtype.lower()=='mysql':
                    import pymysql
                  
                    self.conn = pymysql.connect(db=self.dbname, host=self.host,
                                                port=int(self.port),
                                                user=self.user,
                                                password=str(self.pwd),
                                                charset='utf8mb4',
                                                connect_timeout=self._timeout)

                elif self.dbtype.lower()=='oracle':
                    self.conn =cx_Oracle.connect(self.user+'/'+self.pwd+'@'+self.dbname)

                elif self.dbtype.lower()=='db2':
                    import ibm_db_dbi
                    self.conn = ibm_db_dbi.connect("PORT="+str(self.port)+";PROTOCOL=TCPIP;", 
                                                   user=self.user,
                                                   password=self.pwd, 
                                                   host=self.host, 
                                                   database=self.dbname)

                self._connect_pool[key]=self.conn


            log.debug("连接成功,获得=>%s"%self.conn)
           


        except cx_Oracle.DatabaseError:
            raise RuntimeError("配置不正确")

    def _db_commit(self):
        try:
            if self.conn:
                self.conn.commit()
        except:
            pass


    def db_close():
        try:
            [con.close() for con in DBHelper._connect_pool.values()]
         
        except  Exception as ee:
            log.debug(ee)

            raise DBError('关闭数据库连接出现异常，请确认')

                
    def db_execute(self, sql,error="ignore-"):

        res=0

        log.debug("执行sql=>%s"%sql)

        self._db_connect()
        cursor=self.conn.cursor()

        try:
            res=cursor.execute(sql)

            if not sql.split()[0].lower().startswith("select"):
                self.conn.commit()

            if sql.split()[0].lower().startswith("select"):
                res=cursor.fetchall()

        except Exception as e:
            if error=='ignore':
                return res

            log.debug("出现异常=>"+str(e))
            traceback.print_exc()

            # raise RuntimeError("sql执行结果报错.")
        finally:

            cursor.close()
            if isinstance(res,int):
                log.debug("执行结果=>%s(生效行数)"%(res,))
            else:
                log.debug("执行结果=>%s"%(res,))  
            log.debug("\n\n")

            return res
        #return sqlresult

  
    @classmethod
    def get_config_template(cls):
        return {

        'dbtype':'mysql',
        'dbname':'auto',
        'host':'127.0.0.1',
        'port':3306,
        'username':'root',
        'password':123456


        }



        

    @classmethod
    def get_dao_gen_rule(cls):
        pass


 
class DBError(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)


if __name__=="__main__":
    
    data={
    'dbtype':'mysql',
    'dbname':'auto',
    'host':'127.0.0.1',
    'port':'3306',
    'password':"123456",
    'username':'root',
    'password':'123456'

    }


    h=DBHelper(data)
    res=h.db_execute("select * from device")





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2019-04-15 09:01:29
# @Author  : Blackstone
# @to      :
import sys

sys.path.append("..")
from morm import service,context


@service("a")
class EstService:
	pass


@service
class EfdaService(object):
	pass



# class Device(object):

# 	@bind_sql("select * from device where serial={0}")
# 	def query_device(serial):
# 		pass
# 	@bind_sql("select * from device")
# 	def query_all_device():
# 		pass
# 	@bind_sql("update device set port={1} where serial={0}")
# 	def update_device(serial,port):
# 		pass

# 	@bind_sql("insert into device(serial,port) values({0},{1})")
# 	def add_device(serial,port):
# 		pass

# 	@bind_sql("delete from device where port={0}")
# 	def del_device(port):
# 		pass

# 	@insert
# 	def  add(deivce):
# 		pass

# 	@update(keys=("serial",))
# 	def update(device):
# 		pass

# 	@delete(keys=("serial",))
# 	def delete(device):
# 		pass


# 	@select(keys=("serial",))
# 	def query(device):
# 		pass



def ff():
	print("name=>",__package__)

if __name__=="__main__":
	pass

	#context.getService("a")
	# a=EstService()

	# b=EfdaService()
	# print(a,b)
	







#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2019-04-25 16:25:52
# @Author  : Blackstone
# @to      :

import sys,os
sys.path.append("..")
from morm import context


def tt():
    print (sys._getframe().f_code.co_filename) #当前文件名，可以通过__file__获得
    print (sys._getframe(0).f_code.co_name ) #当前函数名
    print (sys._getframe(1).f_code.co_name)#调用该函数的函数的名字，如果没有被调用，则返回<module>，貌似call stack的栈底
    print (sys._getframe().f_lineno) #当前行号


def ta():
	tt()

if __name__=="__main__":

	context.init()


	#context.getService("service1")





import os

from mt import lu
log=lu.getLogger("MeORM")

root_path=os.path.dirname(os.path.dirname(__file__))
config_path=os.sep.join([root_path,"config"])
test_path=os.sep.join([root_path,"test"])


#
from collections import namedtuple
Rule =namedtuple("Rule",['tablename','support','keys'])

def upcasefirst(str_):
	return str_[0].upper()+str_[1:]


def lowcasefirst(str_):
	return str_[0].lower()+str_[1:]




def get_connect_config(config_path=None,**configoptions):
	from ._gen import Gen
	from .dbhelper import DBHelper
	config=None

	size=len(configoptions.keys())
	if size>0:
		log.debug("设置临时配置=>")
		log.debug(configoptions)
		
		tmp=DBHelper.get_config_template()
		tmpkeys=tmp.keys()

		for key in configoptions:
			if key in tmpkeys:
				tmp[key]=configoptions[key]

		return tmp;


	else:

		config=Gen.get_config(config_path)

		if config is None:
			config=DBHelper.get_config_template()

			Gen.log("--------------")
			Gen.log("正通过模板方法['get_config_template']连接库.")
			Gen.log("--------------")
			Gen.log("获得模板数据为=> ",config)


	return config





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2019-05-24 14:48:20
# @Author  : Blackstone
# @to      :


def get_connect_config(config_path=None,**configoptions):
	pass



get_connect_config("aa",aa=1)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2019-04-15 09:12:25
# @Author  : Blackstone
# @to      :


from .core.dbhelper import DBHelper
from .core.context import context
from .core.decorator import bind_sql,insert,update,delete,select,service
from .core._gen import Gen





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2019-04-12 09:45:24
# @Author  : Blackstone
# @to      :

import yaml,datetime,os,sys,shutil
from collections import namedtuple
from . import log,config_path,test_path,root_path,upcasefirst,Rule,lowcasefirst

from .dbhelper import DBHelper
from  . import get_connect_config
import warnings


class Gen:

	_Debug=True
	_rulefile=None


	@classmethod
	def set_debug(cls,b):
		cls._Debug=b

	@classmethod
	def log(cls,*msg,**kw):
		if cls._Debug:

			level="Debug"

			v=kw.get("level",None)
			if v:level=v
			cur=str(datetime.datetime.now())[:19]

			msg="".join([str(x) for x in msg])
			print("%s-[%s][M-orm]-|| %s"%(cur,level,msg))


	@classmethod
	def gen(cls,dao_file=None,rule_file=None):

		cls._rulefile=rule_file

		if dao_file is None:
			dao_file=os.sep.join([get_desktop(),"dao"])

		else:
			if not os.path.isdir(dao_file):
				raise TypeError("path路径是一个目录")

			dao_file=os.sep.join([dao_file,"dao"])

		if os.path.exists(dao_file):
			shutil.rmtree(dao_file)  


		log.debug("----------------------")
		log.debug("----开始生成Dao=>[%s]-----"%dao_file)
		log.debug("----------------------")
		os.mkdir(dao_file)

		#__init__.py

		with open(os.sep.join([dao_file,"__init__.py"]),"w"):
			pass


		#

		map_=cls._gen_rules_code()

		for _ in map_:
			
			daofile=os.sep.join([dao_file,_])
			with open(daofile,"w") as f:
				f.write(map_[_])


		log.debug("--生成dao结束.")



	@classmethod
	def _gen_rules_code(cls):

		res={}
		path=""

		rules=cls.get_dao_rules()
		for rule in rules:

			code=""
			prefix="\n".join([
				"from Lib.decorator import bind_sql,insert,update,delete,select",
				"from Lib._gen import Gen",
				"from Lib.dbhelper import DBHelper"

				])

			tablename=rule.tablename
			support=rule.support
			keys=rule.keys
			for s in support:

				method_name=lowcasefirst(tablename)
				bind_sql=""
				arg_str=""


				keystr="keys='%s'"%keys

				if isinstance(keys,(tuple,list)):
					keystr="keys=%s"%keys



				if s=="insert":
					bind_sql="@insert(keys='%s')"%keys
					method_heard="def add_%s(%s)"%(method_name,arg_str)

				elif s=="delete":
					bind_sql="@delete(keys='%s')"%keys
					method_heard="def remove_%s(%s)"%(method_name,arg_str)

				elif s=="update":
					bind_sql="@update(keys='%s')"%keys
					method_heard="def update_%s(%s)"%(method_name,arg_str)

				elif s=="select":
					bind_sql="@select(keys='%s')"%keys
					method_heard="def query_%s(%s)"%(method_name,arg_str)


				code+="\t"+bind_sql+"\n\t"+method_heard+":\n\t\tpass"+"\n\n"

			#add bind_sql code
			bind_sql="@bind_sql('select * from tablename where param0={0}')"
			method_heard="def func(*params)"
			code+="\t"+bind_sql+"\n\t"+method_heard+":\n\t\tpass"+"\n\n"


			res[".".join([tablename,"py"])]=prefix+"\n\nclass %s(object)\n"%tablename+ code


		return res



	@classmethod
	def _read_config(cls,path=None):

		config_file=None

		if path:
			config_file=path

		else:
			config_file=os.sep.join([config_path,"connect.yaml"])

		if not os.path.exists(config_file):
			warnings.warn("配置文件[%s]不存在,尝试模板方法连接库."%config_file)
			return None


		with open(config_file) as f:
			cls.config=yaml.load(f.read())


			log.debug("----------------------")
			log.debug("---通过配置文件[%s]连接库-----"%config_file)
			log.debug("----------------------")

			log.debug(cls.config)
		
			return cls.config

	@classmethod
	def get_config(cls,path=None):
		return Gen._read_config(path) if not hasattr(Gen,"config") else cls.config



	@classmethod
	def get_dao_rules(cls):

		d=[]

		log.debug("----------------------")
		log.debug("----获取dao生成规则-----")
		log.debug("----------------------")

		if hasattr(Gen,"rules"):
			return cls.rules


		else:
			if cls._rulefile is None:
				cls._rulefile= os.sep.join([config_path,"dao_gen.yaml"])

			if not os.path.exists(cls._rulefile):
				warnings.warn("规则文件[%s]不存在"%cls._rulefile,RuntimeWarning)

			with open(cls._rulefile) as f:
				for r in yaml.load(f.read())["rules"]:
					#print("rule=>",r)
					rule=Rule(r["tablename"], r["support"], r["keys"])

					d.append(rule)


			cls.rules=d

			return cls.rules


	@classmethod
	def get_tabel_column_map(cls):
		d={}
		rules=cls.get_dao_rules()

		if isinstance(rules,list):

			for x in rules:
				d[x.tablename]=Gen._get_table_columns(x.tablename)


		else:
			d[set_.tablename]=Gen._get_table_columns(set_.tablename)



		return d

	@classmethod
	def get_table_keys(cls,tablename):

		rules= cls.get_dao_rules()

		for r in rules:
			if tablename==r.tablename:
				return r.keys


		warnings.warn("表[%s]没有配置keys"%tablename)


		return None


	


	@classmethod
	def _get_table_columns(cls,tablename):


		dbname=get_connect_config().get("dbname")
		sql="select COLUMN_NAME from information_schema.COLUMNS where table_name = '%s' and table_schema = '%s'"%(tablename,dbname)

		log.debug("查找表字段 =>",sql)
		h=dbhelper.DBHelper()
		res=h.db_execute(sql)

		return [x[0] for x in res]




def get_desktop():
	import winreg
	key = winreg.OpenKey(winreg.HKEY_CURRENT_USER,r'Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders')#利用系统的链表
	return str(winreg.QueryValueEx(key, "Desktop")[0] )#返回的是Unicode类型数据


if __name__=="__main__":
	print(get_desktop())






from distutils.core import setup
from setuptools import find_packages
#导入setup函数
 
setup(
	  name="MeORM", 
	  version="1.47",
	  description="orm框架",
	  author="blackstone",
	  author_email="971406187@qq.com",
	  url="https://github.com/Blackstone1204",
	  #py_modules=['morm']
	  packages=find_packages(),
	  package_data = {
	  # If any package contains *.txt or *.rst files, include them:
	  '': ['*.yaml']
	  # include any *.msg files found in the 'hello' package, too:
	  # 'config': ['*.yaml']
		}

	  )


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Date    : 2019-04-25 13:40:44
# @Author  : Blackstone
# @to      :
from contextlib import contextmanager
import os,sys

from warnings import warn
from . import log


class context(object):
	_service_map={}

	_ready=False#true才能注册
	_ok=False#ok才能获取


	@classmethod
	def _set_ready(cls,bool_=None):

		if bool_ is None:
			cls._ready=True

		else:
			cls._ready=bool_



	@classmethod
	def _is_ready(cls):
		return cls._ready


	@classmethod
	def _is_ok(cls):
		return cls._ok

	@classmethod
	def _set_ok(cls):
		cls._ok=True



	@classmethod
	def getService(cls,name):

		if not cls._is_ok():
			return

		serivce=cls._service_map.get(name,None)
		if serivce is None:
			# raise KeyError("通过[%s]查询服务实例 实例不存在."%name)
			pass

		log.debug("通过[名称-%s]获取到服务实例=>  %s"%(name,serivce))

		return serivce


	@classmethod
	def register_service(cls,name,service):

		if not cls._is_ready():
			return

		log.debug("注册服务实例%s 名称=%s"%(service,name))
		old=cls._service_map.get(name)
		if old is not None:
			warn("服务名已被注册=>%s"%name)

		cls._service_map[name]=service

	@classmethod
	def init(cls,service_root=None):
		"""
		service_root:服务发现位置 默认init调用模块平行目录

		"""

		if service_root is None:
			service_root=os.path.dirname(sys._getframe(1).f_code.co_filename)

		print("---------------")


		log.debug("=============容器开始初始化...")

	


		res=cls._scan_service(service_root)

		cls._set_ready()
		#烧苗结束 注册服务
		for m in res:
			try:
				log.debug("载入服务模块=> %s "%m)
				__import__(m)
			except ImportError as e:
				log.error("容器初始化失败..")
				log.error("%s处未发现模块 %s，载入失败."%(__file__,service_root))

				log.error(str(e))

				return

		cls._set_ready(False)

		cls._set_ok()

		log.debug("==========容器初始化结束..\n")


		#可以获取服务

	@classmethod
	def _scan_service(cls,path,*args,**kw):

		def _scan(file_path):
			#print("file=>",file_path,os.path.isdir(file_path))

			abspath=os.path.abspath(file_path)

			#print("full_name=>",file_path)

			if os.path.isdir(file_path):

				package_sep.append(os.path.splitext(file_path)[0].split(os.sep)[-1])

				#print("package_sep=>",package_sep)

				for f in os.listdir(file_path) :
					
					full_name=abspath+os.sep+f

					if full_name.__contains__("service"):
						_scan(full_name)

				#
				package_sep.clear()
				
			else:
				ext=os.path.splitext(file_path)[-1]
				#print("ext=>",ext)


				packagename=".".join(package_sep)

				modulename=os.path.splitext(file_path)[0].split(os.sep)[-1]

				if ext==".py":
					result.append(packagename+"."+ modulename)


		#_scan_path=(lambda path:os.path.dirname(__file__) if path is None else path)(path)

		log.debug("开始serivce扫描.")
		log.debug("扫描位置 => %s"%path)

		result=[]

		package_sep=[]

		_scan(path)
		result=[".".join(x.split(".")[1:]) for x in result]
		log.debug("扫描结果 =>%s"%result)

		if len(result)==0:
			log.debug("未发现任何可载入的服务 请检查配置.")

		return result



if __name__=="__main__":

	context.init(r"C:\Users\F\Desktop\test")
	
	#print(dot_name_to_object("."))

